//
// CYBERSOURCE CORPORATION ("COMPANY") CONFIDENTIAL
// Copyright (c) 2021 CyberSource Corporation. All Rights Reserved.
//
// NOTICE:  All information contained herein is, and remains the property of COMPANY. The intellectual and technical concepts contained
// herein are proprietary to COMPANY and may be covered by European or foreign Patents, patents in process, and are protected by trade secret or copyright law.
// Dissemination of this information or reproduction of this material is strictly forbidden unless prior written permission is obtained
// from COMPANY.  Access to the source code contained herein is hereby forbidden to anyone except current COMPANY employees, managers or contractors who have executed
// Confidentiality and Non-disclosure agreements explicitly covering such access.
//
// The copyright notice above does not evidence any actual or intended publication or disclosure of this source code, which includes
// information that is confidential and/or proprietary, and is a trade secret, of COMPANY.
// ANY REPRODUCTION, MODIFICATION, DISTRIBUTION, PUBLIC  PERFORMANCE,
// OR PUBLIC DISPLAY OF OR THROUGH USE  OF THIS  SOURCE CODE  WITHOUT  THE EXPRESS WRITTEN CONSENT OF COMPANY IS STRICTLY PROHIBITED, AND IN VIOLATION OF APPLICABLE
// LAWS AND INTERNATIONAL TREATIES.  THE RECEIPT OR POSSESSION OF  THIS SOURCE CODE AND/OR RELATED INFORMATION DOES NOT CONVEY OR IMPLY ANY RIGHTS
// TO REPRODUCE, DISCLOSE OR DISTRIBUTE ITS CONTENTS, OR TO MANUFACTURE, USE, OR SELL ANYTHING THAT IT  MAY DESCRIBE, IN WHOLE OR IN PART.

#import <mpos_core/MPAccessory.h>
#import <mpos_core/MPReceipt.h>
#import <mpos_core/MPTransaction.h>

@protocol MPProviderComponentDelegate;
@protocol MPAccessoryComponentDelegate;

@class MPAccessoryUpdateRequirement;
@class MPTransactionActionSupport;
@class MPPaymentDetailsFactory;
@class MPTransactionActionResponse;
@class MPTransactionActionResponseFactory;
@class MPTransactionParameters;
@class MPAccessoryParameters;
@class MPTransactionFilterParameters;
@class MPAccountParameters;

/**
 * Success handler for connecting to an accessory (e.g. PED or a printer).
 * @param accessory The connected accessory
 * @since 2.0.0
 */
typedef void (^MPAccessoryConnectSuccess)(MPAccessory *_Nonnull accessory);
/**
 * Failure handler for connecting to an accessory (e.g. PED or a printer).
 * @param error Error describing what happened
 * @since 2.0.0
 */
typedef void (^MPAccessoryConnectFailure)(NSError *_Nonnull error);


/**
 * Success handler for checking if an accessory must be updated before it can be used.
 * @param accessory The accessory that was checked
 * @param updateRequirement Indicating the update requirement
 * @since 2.0.0
 */
typedef void (^MPAccessoryCheckUpdateSuccess)(MPAccessory *_Nonnull accessory, MPAccessoryUpdateRequirement *_Nonnull updateRequirement);
/**
 * Failure handler for checking if an accessory must be updated before it can be used.
 * @param accessory The accessory that failed to update
 * @param error Error describing why the update failed
 * @since 2.0.0
 */
typedef void (^MPAccessoryCheckUpdateFailure)(MPAccessory *_Nonnull accessory, NSError *_Nonnull error);


/**
 * Success handler for updating an accessory.
 * @param accessory The accessory that was updated
 * @since 2.0.0
 */
typedef void (^MPAccessoryUpdateSuccess)(MPAccessory *_Nonnull accessory);
/**
 * Failure handler for updating an accessory.
 * @param accessory The accessory that failed to update
 * @param error Error describing why the update failed
 * @since 2.0.0
 */
typedef void (^MPAccessoryUpdateFailure)(MPAccessory *_Nonnull accessory, NSError *_Nonnull error);

/**
 * Success handler for provisioning an accessory.
 * @param accessory The accessory that was provisioned
 * @since 2.11.0
 */
typedef void (^MPAccessoryProvisionSuccess)(MPAccessory *_Nonnull accessory);
/**
 * Failure handler for provisioning an accessory.
 * @param accessory The accessory that failed to be provisioned
 * @param error Error describing why the provisioning failed
 * @since 2.11.0
 */
typedef void (^MPAccessoryProvisionFailure)(MPAccessory *_Nonnull accessory, NSError *_Nonnull error);

/**
 * Success handler for updating accessory state
 * @param accessory The accessory with updated state
 */
typedef void (^MPAccessoryUpdateStateSuccess)(MPAccessory *_Nonnull accessory);

/**
 * Failure handler for updating accessory state
 * @param accessory The accessory whose state failed to be updated
 * @param error Error describing why the update failed
 */
typedef void (^MPAccessoryUpdateStateFailure)(MPAccessory *_Nonnull accessory, NSError *_Nonnull error);

/**
 * Success handler for disconnection from an accessory.
 * @param accessory The accessory that was disconnected - the reference is no longer valid for interactions
 * @since 2.0.0
 */
typedef void (^MPAccessoryDisconnectSuccess)(MPAccessory *_Nonnull accessory);
/**
 * Failure handler for disconnection from an accessory.
 * @param accessory The accessory that should have been disconnected
 * @param error Error describing why the disconnect failed
 * @since 2.0.0
 */
typedef void (^MPAccessoryDisconnectFailure)(MPAccessory *_Nonnull accessory, NSError *_Nonnull error);


/**
 * Success handler for a transaction lookup with our gateway.
 * @param transaction The requested transaction
 * @since 2.0.0
 */
typedef void (^MPTransactionLookupSuccess)(MPTransaction *_Nonnull transaction);
/**
 * Failure handler for a transaction lookup with our gateway.
 * @param identifier The identifier used to lookup the transaction
 * @param error Error describing why the lookup failed
 * @since 2.0.0
 */
typedef void (^MPTransactionLookupFailure)(NSString *_Nonnull identifier, NSError *_Nonnull error);

/**
 * Approval handler for executing a tranaction. At this stage the transaction is approved (completed successfully, executed and approved by the host).
 * @param transaction The approved transaction
 * @since 2.0.0
 */
typedef void (^MPTransactionApproval)(MPTransaction *_Nonnull transaction);

/**
 * Accept handler for executing an offline tranaction. At this stage the transaction is accepted offline and is ready to be updated with the next batch
 * to be executed by the host.
 * @param transaction The accepted transaction
 * @since 2.14.0
 */
typedef void (^MPTransactionAccept)(MPTransaction *_Nonnull transaction);

/**
 * Decline handler for executing a tranaction. At this stage the transaction is declined (completed successfully, executed but declined by the host or terminal).
 * @param transaction The declined transaction
 * @since 2.0.0
 */
typedef void (^MPTransactionDecline)(MPTransaction *_Nonnull transaction);

/**
 * Abort handler for executing a tranaction. At this stage the transaction is aborted.
 * @param transaction The declined transaction
 * @since 2.0.0
 */
typedef void (^MPTransactionAbort)(MPTransaction *_Nonnull transaction);

/**
 * Failure handler for the execution of a transaction. There are different reasons why a transaction may fail: network errors, missing parameters, invalid payment details). Some of
 * those errors are recoverable (e.g. by just starting the transaction again). The error will provide information about the recoverability. The error is also attached to the transaction as an error property, giving access to it at a later stage.
 * @param transaction The failed transaction
 * @param error Indicates what happened and how to proceed
 * @since 2.0.0
 */
typedef void (^MPTransactionFailure)(MPTransaction *_Nonnull transaction, NSError *_Nonnull error);

/**
 * Success handler for transaction capture.
 *
 * @param transaction The captured transaction
 * @since 2.8.0
 */
typedef void (^MPTransactionCaptureSuccess)(MPTransaction *_Nonnull transaction);

/**
 * Failure handler for transaction capture.
 *
 * @param error Describes why the refund failed (if it failed due to an internal error)
 * @since 2.8.0
 */
typedef void (^MPTransactionCaptureFailure)(NSError *_Nonnull error);

/**
 * Action handler for a transaction, used to request additional details when needed (e.g. some POS transaction needs a customer signature).
 * To only get requests for actions that you support, provide a bitmask of actions when initializing the provider.
 * @param transaction The transaction requesting the additional action/data
 * @param action The requested action/data
 * @param supportData Additional data that may be needed for certain types of actions (will require a downcast)
 * @since 2.0.0
 */
typedef void (^MPTransactionActionRequired)(MPTransaction *_Nonnull transaction, MPTransactionAction action, MPTransactionActionSupport *_Nullable supportData);


/**
 * Handler indicating a successful abort of a transaction.
 * @param transaction The aborted transaction
 * @since 2.0.0
 */
typedef void (^MPTransactionAbortSuccess)(MPTransaction *_Nonnull transaction);

/**
 * Handler indicating an error during an attempted abort of a transaction. If this is called, the transaction was NOT aborted.
 * @param transaction The transaction that should have been aborted
 * @param error The error that occured while aborting the transaction
 * @since 2.0.0
 */
typedef void (^MPTransactionAbortFailure)(MPTransaction *_Nonnull transaction, NSError *_Nonnull error);

/**
 * Handler indicating that the tansaction that wants to be aborted has already been aproved. If this is called, the transaction was NOT aborted.
 * @param transaction The transaction that should have been aborted
 * @since 2.52.0
 */
typedef void (^MPTransactionAbortApproved)(MPTransaction *_Nullable transaction);

/**
 * Handler indicating an approved refund of a transaction.
 * @param transaction The refunded transaction
 * @since 2.3.0
 */
typedef void (^MPRefundTransactionWithoutCardApproved)(MPTransaction *_Nonnull transaction);

/**
 * Handler indicating a declined refund of a transaction.
 * @param transaction The refunded transaction
 * @since 2.3.0
 */
typedef void (^MPRefundTransactionWithoutCardDeclined)(MPTransaction *_Nonnull transaction);

/**
 * Handler indicating an error during an attempted refund without card. If this is called, the transaction was NOT refunded:
 * @param transactionParameters The transaction parameters that should have been refunded
 * @param error The error that occured while refunding the transaction
 * @since 2.5.0
 */
typedef void (^MPRefundTransactionParamsWithoutCardFailure)(MPTransactionParameters *_Nonnull transactionParameters, NSError *_Nonnull error);


/**
 * Handler indicating an approved refund of a transaction.
 * @param transaction The original (charge) transaction that was refunded
 * @since 2.8.0
 */
typedef void (^MPRefundTransactionApproved)(MPTransaction *_Nonnull transaction);

/**
 * Handler indicating a declined refund of a transaction.
 * @param transaction The original (charge) transaction that was refunded
 * @since 2.8.0
 */
typedef void (^MPRefundTransactionDeclined)(MPTransaction *_Nonnull transaction);

/**
 * Handler indicating an error during an attempted refund. If this is called, the transaction was NOT refunded:
 * @param transactionParameters The transaction parameters that should have been refunded
 * @param error The error that occured while refunding the transaction
 * @since 2.8.0
 */
typedef void (^MPRefundTransactionFailure)(MPTransactionParameters *_Nonnull transactionParameters, NSError *_Nonnull error);


/**
 * Success handler for tip adjust transaction.
 *
 * @param transaction The tip adjust transaction
 * @since 2.22.0
 */
typedef void (^MPTransactionTipAdjustSuccess)(MPTransaction *_Nonnull transaction);

/**
 * Failure handler for tip adjust transaction.
 *
 * @param transactionParameters The tip adjust transaction parameters
 * @param error Describes why the tip adjust transaction failed (if it failed due to an internal error)
 * @since 2.22.0
 */
typedef void (^MMPTransactionTipAdjustFailure)(MPTransactionParameters *_Nonnull transactionParameters, NSError *_Nonnull error);


/**
 * Success handler for sending a receipt via email.
 * @since 2.3.0
 */
typedef void (^MPCustomerReceiptSendingSuccess)(NSString *_Nonnull transactionIdentifier, NSString *_Nonnull emailAddress);

/**
 * Failure handler for sending a receipt via email.
 * @param error Error describing why the mail sending failed
 * @since 2.3.0
 */
typedef void (^MPCustomerReceiptSendingFailure)(NSString *_Nonnull transactionIdentifier, NSString *_Nonnull emailAddress, NSError *_Nonnull error);

/**
 * Success handler for a transaction query
 * @param transactions The returned transactions (empty if no transcations where found)
 * @since 2.6.0
 */
typedef void (^MPTransactionQuerySuccess)(NSArray<MPTransaction *> *_Nonnull transactions);

/**
 * Failure handler for a transaction query
 * @param error Error describing why query failed
 * @since 2.6.0
 */
typedef void (^MPTransactionQueryFailure)(NSError *_Nonnull error);


/**
 * Success handler for creating a new transaction and registering it within our systems.
 * @param transaction The requested transaction
 * @since 2.0.0
 */
typedef void (^MPTransactionCreateSuccess)(MPTransaction *_Nonnull transaction);

/**
 * Failure handler for creating a new transaction and registering it within our systems.
 * @param error Error describing why it failed
 * @since 2.0.0
 */
typedef void (^MPTransactionCreateFailure)(NSError *_Nonnull error);


/**
 * Indicating the mode the provider will run in.
 * @since 2.0.0
 */

typedef NS_ENUM(NSUInteger, MPProviderMode) {
    /** Unkndown or not available */
    MPProviderModeUnknown = 0,
    /** Live transactions */
    MPProviderModeLIVE,
    /** Test transactions */
    MPProviderModeTEST,
    /** Mocked transactions */
    MPProviderModeMOCK,
    /** Live transactions routed through a fixed address range */
    MPProviderModeLIVE_FIXED,
    /** Test transactions routed through a fixed address range */
    MPProviderModeTEST_FIXED,
};


/**
 * Reasons why a transaction can be aborted
 * @since 2.11.0
 */
typedef NS_ENUM(NSUInteger, MPTransactionAbortReason) {

    /** The merchant aborted the transaction. */
    MPTransactionAbortReasonMerchantAborted = 0,
    /** The accessory encountered an error. */
    MPTransactionAbortReasonAccessoryError,
    /** The accessory is not whitelisted and (possibly) could not be provisioned. */
    MPTransactionAbortReasonAccessoryNotWhitelisted,
};

/**
 * The payment provider is the main instance responsible for handling, creating and managing transactions and the connection to external accessories.
 * @since 2.0.0
 */
@interface MPProvider : NSObject

/**
 * Supported actions as provided via provider options.
 * @since 2.0.0
 */
@property (assign, readonly, nonatomic) MPTransactionAction supportedActions;

/**
 * The mode the provider is running in.
 * @since 2.0.0
 */
@property (assign, readonly, nonatomic) MPProviderMode providerMode;

/**
 * Returns a factory that must be used to generate the payment parameters when making a transaction.
 * @since 2.0.0
 */
@property (strong, readonly, nonatomic, nonnull) MPPaymentDetailsFactory *paymentDetailsFactory __attribute__ ((deprecated("use paymentDetails instead")));

/**
 * Returns a payment details instance to provide the details of a payment
 * @since 2.61.0
 */
@property (strong, readonly, nonatomic, nonnull) MPPaymentDetails *paymentDetails;

/**
 * Returns a factory that must be used to generate action responses during transactions.
 * @since 2.0.0
 */
@property (strong, readonly, nonatomic, nonnull) MPTransactionActionResponseFactory *transactionActionResponseFactory;

/**
 * Returns a set containing all accessories that are currently handled by the provider (either connected/disconnected/not in range).
 * @since 2.0.0
 */
@property (strong, readonly, nonatomic, nonnull) NSSet<MPAccessory *> *accessories;

/**
 * Dispatch queue used for block callbacks. If not set (NULL), dispatch_get_main_queue() will be used.
 * @since 2.0.0
 */
@property (strong, nonatomic, nonnull) dispatch_queue_t callbackQueue;

/**
 * The max line length for the receipt, should be set to the amount of characters your printer is able to print (not relevant for digital receipts), 0 is the default value (uses backend default)
 * @since 2.42.0
 */
@property (assign, nonatomic) NSUInteger maxReceiptLineLength;

#pragma mark -
#pragma mark Handle Accessories
/**
 * @name Handle Accessories
 */

/**
 * Connects to an accessory with the given parameters.
 * @param parameters The options to be used for the connection effort
 * @param success The success handler called when the connection is made
 * @param failure The failure handler called when no connection is possible
 * @throws NSException If the options are invalid
 * @since 2.5.0
 */
- (void)connectToAccessoryWithParameters:(nonnull MPAccessoryParameters *)parameters
                                 success:(nonnull MPAccessoryConnectSuccess)success
                                 failure:(nonnull MPAccessoryConnectFailure)failure;


/**
 * Checks if an update for the given accessory is available and must be installed before a transaction can take place.
 * @param accessory The accessory to check
 * @param success The success handler called when the connection is made
 * @param failure The failure handler called when no connection is possible
 * @throws NSException if the accessory is invalid
 * @since 2.0.0
 */
- (void)checkUpdateRequirementForAccessory:(nonnull MPAccessory *)accessory
                                   success:(nonnull MPAccessoryCheckUpdateSuccess)success
                                   failure:(nonnull MPAccessoryCheckUpdateFailure)failure;


/**
 * Updates the accessory to the configuration/software.
 * @param accessory The accessory to update
 * @param success The success handler called when the accessory was successfully updated or no update was necessary
 * @param failure The failure handler called when the update failed
 * @throws NSException If the accessory is invalid
 * @since 2.0.0
 */
- (void)updateAccessory:(nonnull MPAccessory *)accessory success:(nonnull MPAccessoryUpdateSuccess)success failure:(nonnull MPAccessoryUpdateFailure)failure;

/**
 * Prepares the accessory to be used by the merchant.
 * Use in case the accessory was not assigned to the merchant prior.
 *
 * @param accessory The accessory to provision
 * @since 2.11.0
 */
- (void)provisionAccessory:(nonnull MPAccessory *)accessory success:(nonnull MPAccessoryProvisionSuccess)success failure:(nonnull MPAccessoryProvisionFailure)failure;

/**
 * Updates the accessory state. The accessory will have its details updated as well as battery state and battery level properties updated
 * and all MPAccessoryComoponentDelegate added for this accessory will receive an accessory:didChangeBatteryState:level: notification
 * @param accessory The accessory to querry for battery state
 * @param success The success handler called when accessory battery state was updated
 * @param failure The failure handler called when accessory battery state failed to update
 * @since 2.0.1
 */
- (void)updateAccessoryState:(nonnull MPAccessory *)accessory success:(nonnull MPAccessoryUpdateStateSuccess)success failure:(nonnull MPAccessoryUpdateStateFailure)failure;

/**
 * Disconnects a given accessory. This is normally required if more than once accessory using the Bluetooth connection consecutively (e.g. fist a PED and then a printer).
 * Also note that the accessory reference is no longer valid for interactions and transactions, but rather only given back as a reference.
 * @param accessory The accessory to disconnect from
 * @param success The success handler called when the accessory was disconnected
 * @param failure The failure handler called when the disconnect was not possible (e.g. invalid reference, ongoing transaction)
 * @throws NSException If the accessory is invalid
 * @since 2.0.0
 */
- (void)disconnectFromAccessory:(nonnull MPAccessory *)accessory success:(nonnull MPAccessoryDisconnectSuccess)success failure:(nonnull MPAccessoryDisconnectFailure)failure;


#pragma mark -
#pragma mark Manage Transactions
/**
 * @name Manage Transactions
 */

/**
 * Queries a previous transaction (including state), typically when the transaction failed due to network errors.
 * @param identifier The reference to the transaction session
 * @param success The success handler called when the transaction lookup was successful
 * @param failure The failure handler called when the lookup failed
 * @throws NSException If the identifier is invalid
 * @since 2.0.0
 */
- (void)lookupTransactionWithSessionIdentifier:(nonnull NSString *)identifier
                                       success:(nonnull MPTransactionLookupSuccess)success
                                       failure:(nonnull MPTransactionLookupFailure)failure;

/**
 * Queries a previous transaction (including state), typically when the transaction failed due to network errors.
 * @param identifier The identifier of the transaction
 * @param success The success handler called when the transaction lookup was successful
 * @param failure The failure handler called when the lookup failed
 * @throws NSException If the identifier is invalid
 * @since 2.3.0
 */
- (void)lookupTransactionWithTransactionIdentifier:(nonnull NSString *)identifier
                                           success:(nonnull MPTransactionLookupSuccess)success
                                           failure:(nonnull MPTransactionLookupFailure)failure;


/**
 * Queries transactions filtered by properties set in filterParameters.
 * The transactions are sorted by time in descending order.
 * @param filterParameters The filtering parameters which are used for querying transactions. Use [MPTransactionFilterParameters emptyFilter] if no filtering is required.
 * @param includeReceipts Defines if the returned transaction objects should have customer and merchant receipt information embedded.
 * @param range The range of transactions to be returned, *index* specifies the offset of the first transaction and *length* limits the number of transactions to return.
 * @param success The success handler called when the query was successful
 * @param failure The failure handler called when the query failed
 * @since 2.6.0
 */
- (void)queryTransactionsUsingFilters:(nonnull MPTransactionFilterParameters *)filterParameters
                      includeReceipts:(BOOL)includeReceipts
                                range:(NSRange)range
                              success:(nonnull MPTransactionQuerySuccess)success
                              failure:(nonnull MPTransactionQueryFailure)failure;

/**
 * Registers a new transaction that can then be used to complete a transaction. This step is normally done via a direct server-to-server call but is exposed for testing and rapid prototyping applications.
 * @param parameters The parameters to use to create the transaction
 * @param success The success handler called when the transaction was registered successfully
 * @param failure The failure handler called when an error occured during registration
 * @throws NSException If the parameters are invalid
 * @since 2.5.0
 */
- (void)registerTransactionWithParameters:(nonnull MPTransactionParameters *)parameters
                                  success:(nonnull MPTransactionCreateSuccess)success
                                  failure:(nonnull MPTransactionCreateFailure)failure;


#pragma mark -
#pragma mark Execute Transactions
/**
 * @name Execute Transactions
 */

/**
 * Starts registering and executing a given transaction with the help of the given accessory. From there on action callbacks will request additional parameters if necessary.
 * This request typically starts a mPOS transaction.
 * @param transaction The transaction to be excecuted
 * @param accessory The accessory to use for the transaction
 * @param approval The approved handler called when the transaction was approved.
 * @param decline The declined handler called when the transaction was declined during the execution.
 * @param abort The abort handler called when the transaction was aborted during the execution.
 * @param failure The failure handler called when the transaction failed, have a look at the error for more details
 * @param actionRequired The action required handler called when additional details are necessary (e.g. a customer signature)
 * @throws NSException If the transaction is invalid
 * @since 2.2.0
 */
- (void)startTransaction:(nonnull MPTransaction *)transaction
          usingAccessory:(nonnull MPAccessory *)accessory
                approval:(nonnull MPTransactionApproval)approval
                 decline:(nonnull MPTransactionDecline)decline
                   abort:(nonnull MPTransactionAbort)abort
                 failure:(nonnull MPTransactionFailure)failure
          actionRequired:(nonnull MPTransactionActionRequired)actionRequired;


/**
 * Execute a transaction that does not require an accessory.
 *
 * @param transaction Transaction which is to be executed.
 * @param accountParameters Transaction parameters for the transaction which is to be executed.
 * @param approval The approved handler called when the transaction was approved.
 * @param decline The declined handler called when the transaction was declined during the execution.
 * @param abort The abort handler called when the transaction was aborted during the execution.
 * @param failure The failure handler called when the transaction failed, have a look at the error for more details
 * @since 2.22.0
 */
- (void)executeTransaction:(nonnull MPTransaction *)transaction
         accountParameters:(nonnull MPAccountParameters *)accountParameters
                  approval:(nonnull MPTransactionApproval)approval
                   decline:(nonnull MPTransactionDecline)decline
                     abort:(nonnull MPTransactionAbort)abort
                   failure:(nonnull MPTransactionFailure)failure;

/**
 * Continues a transaction for a given action request, providing additional data if needed. Note that callbacks are still handled by the failure block you handed to startTransaction
 * @param transaction The transaction to continue
 * @param action The action you respond to
 * @param response Response data provided for the specific action (e.g. containing a signature)
 * @throws NSException If the transaction is invalid
 * @since 2.0.0
 */
- (void)continueTransaction:(nonnull MPTransaction *)transaction withAction:(MPTransactionAction)action response:(nullable MPTransactionActionResponse *)response;


/**
 * Attempts to abort a transactions. This is only possible until a certain point of the transaction.
 * If the abort is successful, then the abort block of the startTransaction method will be executed before the success block from this method
 * @note: This is a convenience method which defaults abort reason to MPTransactionAbortReasonMerchantAborted.
 * @param transaction The transaction to abort
 * @param success The success handler called when the transaction was successfully aborted
 * @param failure The failure handler called when aborting the transaction failed.
 * @throws NSException If the transaction is invalid
 * @since 2.0.0
 */
- (void)abortTransaction:(nonnull MPTransaction *)transaction success:(nonnull MPTransactionAbortSuccess)success failure:(nonnull MPTransactionAbortFailure)failure;

/**
 * Attempts to abort a transactions. This is only possible until a certain point of the transaction.
 * If the abort is successful, then the abort block of the startTransaction method will be executed before the success block from this method
 * @note: This is a convenience method which defaults abort reason to MPTransactionAbortReasonMerchantAborted.
 * @param transaction The transaction to abort
 * @param success The success handler called when the transaction was successfully aborted
 * @param failure The failure handler called when aborting the transaction failed.
 * @throws NSException If the transaction is invalid
 * @since 2.0.0
 */
- (void)abortTransaction:(nonnull MPTransaction *)transaction
                 success:(nonnull MPTransactionAbortSuccess)success
                 failure:(nonnull MPTransactionAbortFailure)failure
                approved:(nonnull MPTransactionAbortApproved)approved;

/**
 * Attempts to abort a transactions. This is only possible until a certain point of the transaction.
 * If the abort is successful, then the abort block of the startTransaction method will be executed before the success block from this method
 * @param transaction The transaction to abort
 * @param success The success handler called when the transaction was successfully aborted
 * @param failure The failure handler called when aborting the transaction failed.
 * @throws NSException If the transaction is invalid
 * @since 2.11.0
 */
- (void)abortTransaction:(nonnull MPTransaction *)transaction
                  reason:(MPTransactionAbortReason)reason
                 success:(nonnull MPTransactionAbortSuccess)success
                 failure:(nonnull MPTransactionAbortFailure)failure;

/**
 * Attempts to abort a transactions. This is only possible until a certain point of the transaction.
 * If the abort is successful, then the abort block of the startTransaction method will be executed before the success block from this method
 * @param transaction The transaction to abort
 * @param success The success handler called when the transaction was successfully aborted
 * @param failure The failure handler called when aborting the transaction failed.
 * @throws NSException If the transaction is invalid
 * @since 2.52.0
 */
- (void)abortTransaction:(nonnull MPTransaction *)transaction
                  reason:(MPTransactionAbortReason)reason
                 success:(nonnull MPTransactionAbortSuccess)success
                 failure:(nonnull MPTransactionAbortFailure)failure
                approved:(nonnull MPTransactionAbortApproved)approved;

/**
 * Capture a transaction which has not been yet captured.
 *
 * @param parameters Transaction parameters for the transaction which is to be captured.
 * @throws NSException If the transactionParameters contains invalid data
 * @since 2.8.0
 */
- (void)captureTransactionWithParameters:(nonnull MPTransactionParameters *)parameters
                                 success:(nonnull MPTransactionCaptureSuccess)success
                                 failure:(nonnull MPTransactionCaptureFailure)failure;


#pragma mark -
#pragma mark Send Receipts

/**
 * Sends out an email receipt for a given transaction.
 * @param transactionIdentifier A reference to the transaction
 * @param emailAddress Email receiver of the receipt
 * @param success The success handler called when the sending was successful
 * @param failure The failure handler called when the sending failed
 * @throws NSException If the transation is invalid
 * @since 2.3.0
 */
- (void)sendCustomerReceiptForTransactionIdentifier:(nonnull NSString *)transactionIdentifier
                                       emailAddress:(nonnull NSString *)emailAddress
                                            success:(nonnull MPCustomerReceiptSendingSuccess)success
                                            failure:(nonnull MPCustomerReceiptSendingFailure)failure;

#pragma mark -
#pragma mark Refund Transation without card


/**
 * Refunds a transaction with the given parameters
 * @param transactionParameters The transaction parameters for refunding
 * @param approved The approved handler called when the refund was approved
 * @param declined The declined handler called when the refund was declined
 * @param failure The failure handler called when the refund failed
 * @throws NSException If the transaction is invalid
 * @since 2.5.0
 */
- (void)refundTransactionWithoutCardForParameters:(nonnull MPTransactionParameters *)transactionParameters
                                         approved:(nonnull MPRefundTransactionWithoutCardApproved)approved
                                         declined:(nonnull MPRefundTransactionWithoutCardDeclined)declined
                                          failure:(nonnull MPRefundTransactionParamsWithoutCardFailure)failure;

/**
 * Refunds a transaction with the given parameters
 * This method returns the original (charge) transaction in the successful callback. The refundDetails of this transaction provide more details about the refund if necessary.
 * @param transactionParameters The transaction parameters for refunding
 * @param approved The approved handler called when the refund was approved
 * @param declined The declined handler called when the refund was declined
 * @param failure The failure handler called when the refund failed
 * @throws NSException If the transaction is invalid
 * @since 2.8.0
 */
- (void)refundTransactionWithParameters:(nonnull MPTransactionParameters *)transactionParameters
                               approved:(nonnull MPRefundTransactionApproved)approved
                               declined:(nonnull MPRefundTransactionDeclined)declined
                                failure:(nonnull MPRefundTransactionFailure)failure;


#pragma mark -
#pragma mark Tip Adjust Transation

/**
 * Adjust tip for a transaction with the given parameters
 * @param transactionParameters The transaction parameters for tip adjust
 * @param success The success handler called when the tip adjust succeed
 * @param failure The failure handler called when the tip adjust failed
 * @throws NSException If the transaction is invalid
 * @since 2.22.0
 */
- (void)tipAdjustTransactionWithParameters:(nonnull MPTransactionParameters *)transactionParameters
                                   success:(nonnull MPTransactionTipAdjustSuccess)success
                                   failure:(nonnull MMPTransactionTipAdjustFailure)failure;


#pragma mark -
#pragma mark Register Callbacks
/**
 * @name Register Callbacks
 */

/**
 * Adds an additional provider delegate.
 * @param delegate The delegate to add
 * @since 2.0.0
 */
- (void)addProviderComponentDelegate:(nonnull id<MPProviderComponentDelegate>)delegate;
/**
 * Removes an existing provider delegate.
 * @param delegate The delegate to remove
 * @since 2.0.0
 */
- (void)removeProviderComponentDelegate:(nonnull id<MPProviderComponentDelegate>)delegate;

/**
 * Adds an additional accessory delegate.
 * @param delegate The delegate to add
 * @since 2.0.0
 */
- (void)addAccessoryComponentCallback:(nonnull id<MPAccessoryComponentDelegate>)delegate;
/**
 * Removes an existing accessory delegate.
 * @param delegate The delegate to remove
 * @since 2.0.0
 */
- (void)removeAccessoryComponentCallback:(nonnull id<MPAccessoryComponentDelegate>)delegate;


@end
